@@ -119,6 +119,9 @@ G_DEFINE_TYPE (BraseroMetadata, brasero_metadata, G_TYPE_OBJECT)
 static GSList *downloading = NULL;
 static GSList *downloaded = NULL;
 
+static gboolean
+brasero_metadata_completed (BraseroMetadata *self);
+
 static int
 brasero_metadata_get_xid (BraseroMetadata *metadata)
 {
@@ -421,521 +424,6 @@ brasero_metadata_cancel (BraseroMetadata *self)
 		priv->error = NULL;
 	}
 }
-
-static gboolean
-brasero_metadata_completed (BraseroMetadata *self)
-{
-	BraseroMetadataPrivate *priv;
-
-	priv = BRASERO_METADATA_PRIVATE (self);
-
-	if (priv->error) {
-		BRASERO_UTILS_LOG ("Operation completed with an error %s", priv->error->message);
-	}
-
-	/* we send a message only if we haven't got a loop (= async mode) */
-	g_object_ref (self);
-	g_signal_emit (G_OBJECT (self),
-		       brasero_metadata_signals [COMPLETED_SIGNAL],
-		       0,
-		       priv->error);
-
-	brasero_metadata_stop (self);
-
-	g_object_unref (self);
-
-	/* Return FALSE on purpose here as it will stop the bus callback 
-	 * It's not whether we succeeded or not. */
-	return FALSE;
-}
-
-static gboolean
-brasero_metadata_thumbnail (BraseroMetadata *self)
-{
-	BraseroMetadataPrivate *priv;
-	gint64 position;
-	gboolean res;
-
-	priv = BRASERO_METADATA_PRIVATE (self);
-
-	/* find the right position and move forward */
-	position = 15 * GST_SECOND;
-	while (position > 0 && position >= priv->info->len)
-		position -= 5 * GST_SECOND;
-
-	if (position <= 0)
-		return FALSE;
-
-	gst_element_set_state (priv->pipeline, GST_STATE_PAUSED);
-
-	priv->snapshot_started = 1;
-	if (position < GST_SECOND)
-		position = GST_SECOND;
-
-	res = gst_element_seek_simple (priv->pipeline,
-				       GST_FORMAT_TIME,
-				       GST_SEEK_FLAG_FLUSH,
-				       position);
-
-	BRASERO_UTILS_LOG ("Seeking forward %i for %s", res, priv->info->uri);
-	if (!res)
-		return brasero_metadata_completed (self);
-
-	g_object_set (priv->snapshot,
-		      "send-messages", TRUE,
-		      NULL);
-
-	return TRUE;
-}
-
-static void
-brasero_metadata_is_seekable (BraseroMetadata *self)
-{
-	GstQuery *query;
-	GstFormat format;
-	gboolean seekable;
-	BraseroMetadataPrivate *priv;
-
-	priv = BRASERO_METADATA_PRIVATE (self);
-
-	priv->info->is_seekable = FALSE;
-	query = gst_query_new_seeking (GST_FORMAT_DEFAULT);
-	if (!gst_element_query (priv->source, query))
-		goto end;
-
-	gst_query_parse_seeking (query,
-				 &format,
-				 &seekable,
-				 NULL,
-				 NULL);
-
-	priv->info->is_seekable = seekable;
-
-end:
-	gst_query_unref (query);
-}
-
-static gboolean
-brasero_metadata_get_mime_type (BraseroMetadata *self)
-{
-	BraseroMetadataPrivate *priv;
-	GstElement *typefind;
-	GstCaps *caps = NULL;
-	GstElement *decode;
-	const gchar *mime;
-
-	priv = BRASERO_METADATA_PRIVATE (self);
-
-	if (priv->info->type) {
-		g_free (priv->info->type);
-		priv->info->type = NULL;
-	}
-
-	/* find the type of the file */
-	decode = gst_bin_get_by_name (GST_BIN (priv->pipeline),
-				      "decode");
-	typefind = gst_bin_get_by_name (GST_BIN (priv->decode),
-					"typefind");
-
-	g_object_get (typefind, "caps", &caps, NULL);
-	if (!caps) {
-		gst_object_unref (typefind);
-		return FALSE;
-	}
-
-	if (gst_caps_get_size (caps) <= 0) {
-		gst_object_unref (typefind);
-		return FALSE;
-	}
-
-	mime = gst_structure_get_name (gst_caps_get_structure (caps, 0));
-	gst_object_unref (typefind);
-
-	BRASERO_UTILS_LOG ("Mime type %s", mime);
-
-	if (!mime)
-		return FALSE;
-
-	if (!strcmp (mime, "application/x-id3"))
-		priv->info->type = g_strdup ("audio/mpeg");
-	else if (!strcmp (mime, "audio/x-wav")) {
-		GstElement *wavparse = NULL;
-		gpointer element = NULL;
-		GstIteratorResult res;
-		GstIterator *iter;
-
-		priv->info->type = g_strdup (mime);
-
-		/* make sure it doesn't have dts inside */
-		iter = gst_bin_iterate_recurse (GST_BIN (priv->decode));
-
-		res = gst_iterator_next (iter, &element);
-		while (res == GST_ITERATOR_OK) {
-			gchar *name;
-
-			name = gst_object_get_name (GST_OBJECT (element));
-			if (name) {
-				if (!strncmp (name, "wavparse", 8)) {
-					wavparse = element;
-					g_free (name);
-					break;
-				}
-				g_free (name);
-			}
-
-			gst_object_unref (element);
-			element = NULL;
-
-			res = gst_iterator_next (iter, &element);
-		}
-		gst_iterator_free (iter);
-
-		if (wavparse) {
-			GstPad *src_pad;
-
-			iter = gst_element_iterate_src_pads (wavparse);
-
-			res = gst_iterator_next (iter, (gpointer *) &src_pad);
-			while (res == GST_ITERATOR_OK) {
-				GstCaps *src_caps;
-
-				src_caps = gst_pad_get_caps (src_pad);
-				if (src_caps) {
-					GstStructure *structure;
-
-					structure = gst_caps_get_structure (src_caps, 0);
-					if (structure) {
-						const gchar *name;
-
-						name = gst_structure_get_name (structure);
-						priv->info->has_dts = (g_strrstr (name, "audio/x-dts") != NULL);
-						if (priv->info->has_dts) {
-							gst_object_unref (src_pad);
-							gst_caps_unref (src_caps);
-							src_pad = NULL;
-							break;
-						}
-					}
-					gst_caps_unref (src_caps);
-				}
-
-				gst_object_unref (src_pad);
-				src_pad = NULL;
-
-				res = gst_iterator_next (iter,  (gpointer *) &src_pad);
-			}
-
-			gst_iterator_free (iter);
-			gst_object_unref (wavparse);
-		}
-
-		BRASERO_UTILS_LOG ("Wav file has dts: %s", priv->info->has_dts? "yes":"no");
-	}
-	else
-		priv->info->type = g_strdup (mime);
-
-	return TRUE;
-}
-
-static gboolean
-brasero_metadata_is_mp3 (BraseroMetadata *self)
-{
-	BraseroMetadataPrivate *priv;
-
-	priv = BRASERO_METADATA_PRIVATE (self);
-
-	if (!priv->info->type
-	&&  !brasero_metadata_get_mime_type (self))
-		return FALSE;
-
-	if (!strcmp (priv->info->type, "audio/mpeg"))
-		return TRUE;
-
-	return FALSE;
-}
-
-static void
-foreach_tag (const GstTagList *list,
-	     const gchar *tag,
-	     BraseroMetadata *self)
-{
-	BraseroMetadataPrivate *priv;
-	priv = BRASERO_METADATA_PRIVATE (self);
-
-	if (!strcmp (tag, GST_TAG_TITLE)) {
-		if (priv->info->title)
-			g_free (priv->info->title);
-
-		gst_tag_list_get_string (list, tag, &(priv->info->title));
-	} else if (!strcmp (tag, GST_TAG_ARTIST)
-	       ||  !strcmp (tag, GST_TAG_PERFORMER)) {
-		if (priv->info->artist)
-			g_free (priv->info->artist);
-
-		gst_tag_list_get_string (list, tag, &(priv->info->artist));
-	}
-	else if (!strcmp (tag, GST_TAG_ALBUM)) {
-		if (priv->info->album)
-			g_free (priv->info->album);
-
-		gst_tag_list_get_string (list, tag, &(priv->info->album));
-	}
-	else if (!strcmp (tag, GST_TAG_GENRE)) {
-		if (priv->info->genre)
-			g_free (priv->info->genre);
-
-		gst_tag_list_get_string (list, tag, &(priv->info->genre));
-	}
-/*	else if (!strcmp (tag, GST_TAG_COMPOSER)) {
-		if (self->composer)
-			g_free (self->composer);
-
-		gst_tag_list_get_string (list, tag, &(self->composer));
-	}
-*/	else if (!strcmp (tag, GST_TAG_ISRC)) {
-		gchar *isrc = NULL;
-		gst_tag_list_get_string (list, tag, &isrc);
-
-		if (isrc)
-			priv->info->isrc = (int) g_ascii_strtoull (isrc, NULL, 10);
-	}
-	else if (!strcmp (tag, GST_TAG_MUSICBRAINZ_TRACKID)) {
-		gst_tag_list_get_string (list, tag, &(priv->info->musicbrainz_id));
-	}
-}
-
-static void
-brasero_metadata_process_pending_tag_messages (BraseroMetadata *self)
-{
-	GstBus *bus;
-	GstMessage *msg;
-	BraseroMetadataPrivate *priv;
-
-	priv = BRASERO_METADATA_PRIVATE (self);
-
-	bus = gst_pipeline_get_bus (GST_PIPELINE (priv->pipeline));
-	while ((msg = gst_bus_pop_filtered (bus, GST_MESSAGE_TAG))) {
-		GstTagList *tags = NULL;
-
-		gst_message_parse_tag (msg, &tags);
-		gst_tag_list_foreach (tags, (GstTagForeachFunc) foreach_tag, self);
-		gst_tag_list_free (tags);
-
-		gst_message_unref (msg);
-	}
-
-	g_object_unref (bus);
-}
-
-static gboolean
-brasero_metadata_success (BraseroMetadata *self)
-{
-	BraseroMetadataPrivate *priv;
-
-	priv = BRASERO_METADATA_PRIVATE (self);
-
-	BRASERO_UTILS_LOG ("Metadata retrieval completed for %s", priv->info->uri);
-
-	/* check if that's a seekable one */
-	brasero_metadata_is_seekable (self);
-
-	if (priv->silence) {
-		priv->silence->end = priv->info->len;
-		priv->info->silences = g_slist_append (priv->info->silences, priv->silence);
-		priv->silence = NULL;
-	}
-
-	/* before leaving, check if we need a snapshot */
-	if (priv->info->len > 0
-	&&  priv->snapshot
-	&&  priv->video_linked
-	&& !priv->snapshot_started)
-		return brasero_metadata_thumbnail (self);
-
-	return brasero_metadata_completed (self);
-}
-
-static gboolean
-brasero_metadata_get_duration (BraseroMetadata *self,
-			       GstElement *pipeline,
-			       gboolean use_duration)
-{
-	GstFormat format = GST_FORMAT_TIME;
-	BraseroMetadataPrivate *priv;
-	gint64 duration = -1;
-
-	priv = BRASERO_METADATA_PRIVATE (self);
-
-	if (!use_duration)
-		gst_element_query_position (GST_ELEMENT (pipeline),
-					    &format,
-					    &duration);
-	else
-		gst_element_query_duration (GST_ELEMENT (pipeline),
-					    &format,
-					    &duration);
-
-	if (duration == -1) {
-		if (!priv->error) {
-			gchar *name;
-
-			BRASERO_GET_BASENAME_FOR_DISPLAY (priv->info->uri, name);
-			priv->error = g_error_new (BRASERO_UTILS_ERROR,
-						   BRASERO_UTILS_ERROR_GENERAL,
-						   _("\"%s\" could not be handled by GStreamer."),
-						   name);
-			g_free (name);
-		}
-
-		return brasero_metadata_completed (self);
-	}
-
-	BRASERO_UTILS_LOG ("Found duration %lli for %s", duration, priv->info->uri);
-
-	priv->info->len = duration;
-	return brasero_metadata_success (self);
-}
-
-/**
- * This is to deal with mp3 more particularly the vbrs
- **/
-
-static gboolean
-brasero_metadata_mp3_bus_messages (GstBus *bus,
-				   GstMessage *msg,
-				   BraseroMetadata *self)
-{
-	BraseroMetadataPrivate *priv;
-	gchar *debug_string = NULL;
-	GError *error = NULL;
-
-	priv = BRASERO_METADATA_PRIVATE (self);
-
-	switch (GST_MESSAGE_TYPE (msg)) {
-	case GST_MESSAGE_ERROR:
-		/* save the error message */
-		gst_message_parse_error (msg, &error, &debug_string);
-		BRASERO_UTILS_LOG ("Gstreamer error - mp3 - (%s)", debug_string);
-		g_free (debug_string);
-		if (!priv->error && error)
-			priv->error = error;
-
-		brasero_metadata_completed (self);
-		return FALSE;
-
-	case GST_MESSAGE_EOS:
-		BRASERO_UTILS_LOG ("End of stream reached - mp3 - for %s", priv->info->uri);
-		brasero_metadata_get_duration (self, priv->pipeline_mp3, FALSE);
-		return FALSE;
-
-	default:
-		break;
-	}
-
-	return TRUE;
-}
-
-static gboolean
-brasero_metadata_create_mp3_pipeline (BraseroMetadata *self)
-{
-	BraseroMetadataPrivate *priv;
-	GstElement *source;
-	GstElement *parse;
-	GstElement *sink;
-	GstBus *bus;
-
-	priv = BRASERO_METADATA_PRIVATE (self);
-
-	priv->pipeline_mp3 = gst_pipeline_new (NULL);
-
-	source = gst_element_make_from_uri (GST_URI_SRC,
-					    priv->info->uri,
-					    NULL);
-	if (!source) {
-		priv->error = g_error_new (BRASERO_UTILS_ERROR,
-					   BRASERO_UTILS_ERROR_GENERAL,
-					   _("%s element could not be created"),
-					   "\"Source\"");
-
-		g_object_unref (priv->pipeline_mp3);
-		priv->pipeline_mp3 = NULL;
-		return FALSE;
-	}
-	gst_bin_add (GST_BIN (priv->pipeline_mp3), source);
-
-	parse = gst_element_factory_make ("mp3parse", NULL);
-	if (!parse) {
-		priv->error = g_error_new (BRASERO_UTILS_ERROR,
-					   BRASERO_UTILS_ERROR_GENERAL,
-					   _("%s element could not be created"),
-					   "\"mp3parse\"");
-
-		g_object_unref (priv->pipeline_mp3);
-		priv->pipeline_mp3 = NULL;
-		return FALSE;
-	}
-	gst_bin_add (GST_BIN (priv->pipeline_mp3), parse);
-
-	sink = gst_element_factory_make ("fakesink", NULL);
-	if (!sink) {
-		priv->error = g_error_new (BRASERO_UTILS_ERROR,
-					   BRASERO_UTILS_ERROR_GENERAL,
-					   _("%s element could not be created"),
-					   "\"Fakesink\"");
-
-		g_object_unref (priv->pipeline_mp3);
-		priv->pipeline_mp3 = NULL;
-		return FALSE;
-	}
-	gst_bin_add (GST_BIN (priv->pipeline_mp3), sink);
-
-	/* Link */
-	if (!gst_element_link_many (source, parse, sink, NULL)) {
-		g_object_unref (priv->pipeline_mp3);
-		priv->pipeline_mp3 = NULL;
-		return FALSE;
-	}
-
-	/* Bus */
-	bus = gst_pipeline_get_bus (GST_PIPELINE (priv->pipeline_mp3));
-	priv->watch_mp3 = gst_bus_add_watch (bus,
-					     (GstBusFunc) brasero_metadata_mp3_bus_messages,
-					     self);
-	gst_object_unref (bus);
-
-	gst_element_set_state (priv->pipeline_mp3, GST_STATE_PLAYING);
-	return TRUE;
-}
-
-static gboolean
-brasero_metadata_success_main (BraseroMetadata *self)
-{
-	BraseroMetadataPrivate *priv;
-
-	priv = BRASERO_METADATA_PRIVATE (self);
-
-	BRASERO_UTILS_LOG ("Metadata retrieval successfully completed for %s", priv->info->uri);
-
-	/* find the type of the file */
-	brasero_metadata_get_mime_type (self);
-
-	/* empty the bus of any pending message */
-	brasero_metadata_process_pending_tag_messages (self);
-
-	/* get the size */
-	if (brasero_metadata_is_mp3 (self)) {
-		if (!brasero_metadata_create_mp3_pipeline (self)) {
-			BRASERO_UTILS_LOG ("Impossible to run mp3 specific pipeline");
-			return brasero_metadata_completed (self);
-		}
-
-		/* Return FALSE here not because we failed but to stop the Bus callback */
-		return FALSE;
-	}
-
-	return brasero_metadata_get_duration (self, priv->pipeline, TRUE);
-}
-
 static void
 brasero_metadata_install_plugins_add_downloaded (GSList *downloads)
 {
@@ -1129,6 +617,7 @@ brasero_metadata_install_missing_plugins (BraseroMetadata *self)
 		 * - is being downloaded
 		 * If so don't do anything. */
 		detail = gst_missing_plugin_message_get_installer_detail (iter->data);
+		gst_mini_object_unref (iter->data);
 
 		download = brasero_metadata_is_downloading (detail);
 		if (download) {
@@ -1153,6 +642,9 @@ brasero_metadata_install_missing_plugins (BraseroMetadata *self)
 		g_ptr_array_add (details, detail);
 	}
 
+	g_slist_free (priv->missing_plugins);
+	priv->missing_plugins = NULL;
+
 	if (!details->len) {
 		/* either these plugins were downloaded or are being downloaded */
 		g_ptr_array_free (details, TRUE);
@@ -1174,7 +666,7 @@ brasero_metadata_install_missing_plugins (BraseroMetadata *self)
 					    downloads);
 
 	gst_install_plugins_context_free (context);
-	g_ptr_array_free (details, FALSE);
+	g_ptr_array_free (details, TRUE);
 
 	BRASERO_UTILS_LOG ("Download status %i", status);
 
@@ -1186,6 +678,614 @@ brasero_metadata_install_missing_plugins (BraseroMetadata *self)
 	return TRUE;
 }
 
+static gboolean
+brasero_metadata_completed (BraseroMetadata *self)
+{
+	BraseroMetadataPrivate *priv;
+
+	priv = BRASERO_METADATA_PRIVATE (self);
+
+	if (priv->error) {
+		BRASERO_UTILS_LOG ("Operation completed with an error %s", priv->error->message);
+	}
+
+	/* See if we have missing plugins */
+	if (priv->missing_plugins) {
+		if (brasero_metadata_install_missing_plugins (self))
+			return TRUE;
+	}
+
+	/* we send a message only if we haven't got a loop (= async mode) */
+	g_object_ref (self);
+	g_signal_emit (G_OBJECT (self),
+		       brasero_metadata_signals [COMPLETED_SIGNAL],
+		       0,
+		       priv->error);
+
+	brasero_metadata_stop (self);
+
+	g_object_unref (self);
+
+	/* Return FALSE on purpose here as it will stop the bus callback 
+	 * It's not whether we succeeded or not. */
+	return FALSE;
+}
+
+static gboolean
+brasero_metadata_thumbnail (BraseroMetadata *self)
+{
+	BraseroMetadataPrivate *priv;
+	gint64 position;
+	gboolean res;
+
+	priv = BRASERO_METADATA_PRIVATE (self);
+
+	/* find the right position and move forward */
+	position = 15 * GST_SECOND;
+	while (position > 0 && position >= priv->info->len)
+		position -= 5 * GST_SECOND;
+
+	if (position <= 0)
+		return FALSE;
+
+	gst_element_set_state (priv->pipeline, GST_STATE_PAUSED);
+
+	priv->snapshot_started = 1;
+	if (position < GST_SECOND)
+		position = GST_SECOND;
+
+	res = gst_element_seek_simple (priv->pipeline,
+				       GST_FORMAT_TIME,
+				       GST_SEEK_FLAG_FLUSH,
+				       position);
+
+	BRASERO_UTILS_LOG ("Seeking forward %i for %s", res, priv->info->uri);
+	if (!res)
+		return brasero_metadata_completed (self);
+
+	g_object_set (priv->snapshot,
+		      "send-messages", TRUE,
+		      NULL);
+
+	return TRUE;
+}
+
+static void
+brasero_metadata_is_seekable (BraseroMetadata *self)
+{
+	GstQuery *query;
+	GstFormat format;
+	gboolean seekable;
+	BraseroMetadataPrivate *priv;
+
+	priv = BRASERO_METADATA_PRIVATE (self);
+
+	priv->info->is_seekable = FALSE;
+
+	/* NOTE: apparently GST_FORMAT_DEFAULT does not work here */
+	query = gst_query_new_seeking (GST_FORMAT_TIME);
+
+	/* NOTE: it works better now on the pipeline than on the source as we
+	 * used to do */
+	if (!gst_element_query (priv->pipeline, query))
+		goto end;
+
+	gst_query_parse_seeking (query,
+				 &format,
+				 &seekable,
+				 NULL,
+				 NULL);
+
+	priv->info->is_seekable = seekable;
+
+end:
+
+	gst_query_unref (query);
+}
+
+static gboolean
+brasero_metadata_get_mime_type (BraseroMetadata *self)
+{
+	BraseroMetadataPrivate *priv;
+	GstElement *typefind;
+	GstCaps *caps = NULL;
+	const gchar *mime;
+
+	priv = BRASERO_METADATA_PRIVATE (self);
+
+	if (priv->info->type) {
+		g_free (priv->info->type);
+		priv->info->type = NULL;
+	}
+
+	/* find the type of the file */
+	typefind = gst_bin_get_by_name (GST_BIN (priv->decode),
+					"typefind");
+
+	g_object_get (typefind, "caps", &caps, NULL);
+	if (!caps) {
+		gst_object_unref (typefind);
+		return FALSE;
+	}
+
+	if (gst_caps_get_size (caps) <= 0) {
+		gst_object_unref (typefind);
+		return FALSE;
+	}
+
+	mime = gst_structure_get_name (gst_caps_get_structure (caps, 0));
+	gst_object_unref (typefind);
+
+	BRASERO_UTILS_LOG ("Mime type %s", mime);
+
+	if (!mime)
+		return FALSE;
+
+	if (!strcmp (mime, "application/x-id3"))
+		priv->info->type = g_strdup ("audio/mpeg");
+	else if (!strcmp (mime, "audio/x-wav")) {
+		GstElement *wavparse = NULL;
+		gpointer element = NULL;
+		GstIteratorResult res;
+		GstIterator *iter;
+
+		priv->info->type = g_strdup (mime);
+
+		/* make sure it doesn't have dts inside */
+		iter = gst_bin_iterate_recurse (GST_BIN (priv->decode));
+
+		res = gst_iterator_next (iter, &element);
+		while (res == GST_ITERATOR_OK) {
+			gchar *name;
+
+			name = gst_object_get_name (GST_OBJECT (element));
+			if (name) {
+				if (!strncmp (name, "wavparse", 8)) {
+					wavparse = element;
+					g_free (name);
+					break;
+				}
+				g_free (name);
+			}
+
+			gst_object_unref (element);
+			element = NULL;
+
+			res = gst_iterator_next (iter, &element);
+		}
+		gst_iterator_free (iter);
+
+		if (wavparse) {
+			GstPad *src_pad;
+
+			iter = gst_element_iterate_src_pads (wavparse);
+
+			res = gst_iterator_next (iter, (gpointer *) &src_pad);
+			while (res == GST_ITERATOR_OK) {
+				GstCaps *src_caps;
+
+				src_caps = gst_pad_get_caps (src_pad);
+				if (src_caps) {
+					GstStructure *structure;
+
+					structure = gst_caps_get_structure (src_caps, 0);
+					if (structure) {
+						const gchar *name;
+
+						name = gst_structure_get_name (structure);
+						priv->info->has_dts = (g_strrstr (name, "audio/x-dts") != NULL);
+						if (priv->info->has_dts) {
+							gst_object_unref (src_pad);
+							gst_caps_unref (src_caps);
+							src_pad = NULL;
+							break;
+						}
+					}
+					gst_caps_unref (src_caps);
+				}
+
+				gst_object_unref (src_pad);
+				src_pad = NULL;
+
+				res = gst_iterator_next (iter,  (gpointer *) &src_pad);
+			}
+
+			gst_iterator_free (iter);
+			gst_object_unref (wavparse);
+		}
+
+		BRASERO_UTILS_LOG ("Wav file has dts: %s", priv->info->has_dts? "yes":"no");
+	}
+	else
+		priv->info->type = g_strdup (mime);
+
+	return TRUE;
+}
+
+static gboolean
+brasero_metadata_is_mp3 (BraseroMetadata *self)
+{
+	BraseroMetadataPrivate *priv;
+
+	priv = BRASERO_METADATA_PRIVATE (self);
+
+	if (!priv->info->type
+	&&  !brasero_metadata_get_mime_type (self))
+		return FALSE;
+
+	if (!strcmp (priv->info->type, "audio/mpeg"))
+		return TRUE;
+
+	return FALSE;
+}
+
+static void
+foreach_tag (const GstTagList *list,
+	     const gchar *tag,
+	     BraseroMetadata *self)
+{
+	BraseroMetadataPrivate *priv;
+	priv = BRASERO_METADATA_PRIVATE (self);
+
+	if (!strcmp (tag, GST_TAG_TITLE)) {
+		if (priv->info->title)
+			g_free (priv->info->title);
+
+		gst_tag_list_get_string (list, tag, &(priv->info->title));
+	} else if (!strcmp (tag, GST_TAG_ARTIST)
+	       ||  !strcmp (tag, GST_TAG_PERFORMER)) {
+		if (priv->info->artist)
+			g_free (priv->info->artist);
+
+		gst_tag_list_get_string (list, tag, &(priv->info->artist));
+	}
+	else if (!strcmp (tag, GST_TAG_ALBUM)) {
+		if (priv->info->album)
+			g_free (priv->info->album);
+
+		gst_tag_list_get_string (list, tag, &(priv->info->album));
+	}
+	else if (!strcmp (tag, GST_TAG_GENRE)) {
+		if (priv->info->genre)
+			g_free (priv->info->genre);
+
+		gst_tag_list_get_string (list, tag, &(priv->info->genre));
+	}
+/*	else if (!strcmp (tag, GST_TAG_COMPOSER)) {
+		if (self->composer)
+			g_free (self->composer);
+
+		gst_tag_list_get_string (list, tag, &(self->composer));
+	}
+*/	else if (!strcmp (tag, GST_TAG_ISRC)) {
+		gchar *isrc = NULL;
+		gst_tag_list_get_string (list, tag, &isrc);
+
+		if (isrc)
+			priv->info->isrc = (int) g_ascii_strtoull (isrc, NULL, 10);
+	}
+	else if (!strcmp (tag, GST_TAG_MUSICBRAINZ_TRACKID)) {
+		gst_tag_list_get_string (list, tag, &(priv->info->musicbrainz_id));
+	}
+}
+
+static gboolean
+brasero_metadata_process_element_messages (BraseroMetadata *self,
+					   GstMessage *msg)
+{
+	BraseroMetadataPrivate *priv;
+
+	priv = BRASERO_METADATA_PRIVATE (self);
+
+	/* This is for snapshot function */
+	if (!strcmp (gst_structure_get_name (msg->structure), "preroll-pixbuf")
+	||  !strcmp (gst_structure_get_name (msg->structure), "pixbuf")) {
+		const GValue *value;
+
+		value = gst_structure_get_value (msg->structure, "pixbuf");
+		priv->info->snapshot = g_value_get_object (value);
+		g_object_ref (priv->info->snapshot);
+
+		BRASERO_UTILS_LOG ("Received pixbuf snapshot sink (%p) for %s", priv->info->snapshot, priv->info->uri);
+
+		/* Now we can stop */
+		return brasero_metadata_completed (self);
+	}
+
+	/* here we just want to check if that's a missing codec */
+	if ((priv->flags & BRASERO_METADATA_FLAG_MISSING)
+	&&   gst_is_missing_plugin_message (msg)) {
+		priv->missing_plugins = g_slist_prepend (priv->missing_plugins, gst_message_ref (msg));
+	}
+	else if (!strcmp (gst_structure_get_name (msg->structure), "level")
+	&&   gst_structure_has_field (msg->structure, "peak")) {
+		const GValue *value;
+		const GValue *list;
+		gdouble peak;
+
+		list = gst_structure_get_value (msg->structure, "peak");
+		value = gst_value_list_get_value (list, 0);
+		peak = g_value_get_double (value);
+
+		/* detection of silence */
+		if (peak < -50.0) {
+			gint64 pos = -1;
+			GstFormat format = GST_FORMAT_TIME;
+
+			/* was there a silence last time we check ?
+			 * NOTE: if that's the first signal we receive
+			 * then consider that silence started from 0 */
+			gst_element_query_position (priv->pipeline, &format, &pos);
+			if (pos == -1) {
+				BRASERO_UTILS_LOG ("impossible to retrieve position");
+				return TRUE;
+			}
+
+			if (!priv->silence) {
+				priv->silence = g_new0 (BraseroMetadataSilence, 1);
+				if (priv->prev_level_mes) {
+					priv->silence->start = pos;
+					priv->silence->end = pos;
+				}
+				else {
+					priv->silence->start = 0;
+					priv->silence->end = pos;
+				}
+			}				
+			else
+				priv->silence->end = pos;
+
+			BRASERO_UTILS_LOG ("silence detected at %lli", pos);
+		}
+		else if (priv->silence) {
+			BRASERO_UTILS_LOG ("silence finished");
+
+			priv->info->silences = g_slist_append (priv->info->silences,
+							       priv->silence);
+			priv->silence = NULL;
+		}
+		priv->prev_level_mes = 1;
+	}
+
+	return TRUE;
+}
+
+static void
+brasero_metadata_process_pending_messages (BraseroMetadata *self)
+{
+	GstBus *bus;
+	GstMessage *msg;
+	BraseroMetadataPrivate *priv;
+
+	priv = BRASERO_METADATA_PRIVATE (self);
+
+	bus = gst_pipeline_get_bus (GST_PIPELINE (priv->pipeline));
+	while ((msg = gst_bus_pop (bus))) {
+		GstTagList *tags = NULL;
+
+		if (GST_MESSAGE_TYPE (msg) == GST_MESSAGE_TAG) {
+			gst_message_parse_tag (msg, &tags);
+			gst_tag_list_foreach (tags, (GstTagForeachFunc) foreach_tag, self);
+			gst_tag_list_free (tags);
+		}
+		else if (GST_MESSAGE_TYPE (msg) == GST_MESSAGE_ELEMENT)
+			brasero_metadata_process_element_messages (self, msg);
+
+		gst_message_unref (msg);
+	}
+
+	g_object_unref (bus);
+}
+
+static gboolean
+brasero_metadata_success (BraseroMetadata *self)
+{
+	BraseroMetadataPrivate *priv;
+
+	priv = BRASERO_METADATA_PRIVATE (self);
+
+	BRASERO_UTILS_LOG ("Metadata retrieval completed for %s", priv->info->uri);
+
+	/* check if that's a seekable one */
+	brasero_metadata_is_seekable (self);
+
+	if (priv->silence) {
+		priv->silence->end = priv->info->len;
+		priv->info->silences = g_slist_append (priv->info->silences, priv->silence);
+		priv->silence = NULL;
+	}
+
+	/* before leaving, check if we need a snapshot */
+	if (priv->info->len > 0
+	&&  priv->snapshot
+	&&  priv->video_linked
+	&& !priv->snapshot_started)
+		return brasero_metadata_thumbnail (self);
+
+	return brasero_metadata_completed (self);
+}
+
+static gboolean
+brasero_metadata_get_duration (BraseroMetadata *self,
+			       GstElement *pipeline,
+			       gboolean use_duration)
+{
+	GstFormat format = GST_FORMAT_TIME;
+	BraseroMetadataPrivate *priv;
+	gint64 duration = -1;
+
+	priv = BRASERO_METADATA_PRIVATE (self);
+
+	if (!use_duration)
+		gst_element_query_position (GST_ELEMENT (pipeline),
+					    &format,
+					    &duration);
+	else
+		gst_element_query_duration (GST_ELEMENT (pipeline),
+					    &format,
+					    &duration);
+
+	if (duration == -1) {
+		if (!priv->error) {
+			gchar *name;
+
+			BRASERO_GET_BASENAME_FOR_DISPLAY (priv->info->uri, name);
+			priv->error = g_error_new (BRASERO_UTILS_ERROR,
+						   BRASERO_UTILS_ERROR_GENERAL,
+						   _("\"%s\" could not be handled by GStreamer."),
+						   name);
+			g_free (name);
+		}
+
+		return brasero_metadata_completed (self);
+	}
+
+	BRASERO_UTILS_LOG ("Found duration %lli for %s", duration, priv->info->uri);
+
+	priv->info->len = duration;
+	return brasero_metadata_success (self);
+}
+
+/**
+ * This is to deal with mp3 more particularly the vbrs
+ **/
+
+static gboolean
+brasero_metadata_mp3_bus_messages (GstBus *bus,
+				   GstMessage *msg,
+				   BraseroMetadata *self)
+{
+	BraseroMetadataPrivate *priv;
+	gchar *debug_string = NULL;
+	GError *error = NULL;
+
+	priv = BRASERO_METADATA_PRIVATE (self);
+
+	switch (GST_MESSAGE_TYPE (msg)) {
+	case GST_MESSAGE_ERROR:
+		/* save the error message */
+		gst_message_parse_error (msg, &error, &debug_string);
+		BRASERO_UTILS_LOG ("GStreamer error - mp3 - (%s)", debug_string);
+		g_free (debug_string);
+		if (!priv->error && error)
+			priv->error = error;
+
+		brasero_metadata_completed (self);
+		return FALSE;
+
+	case GST_MESSAGE_EOS:
+		BRASERO_UTILS_LOG ("End of stream reached - mp3 - for %s", priv->info->uri);
+		brasero_metadata_get_duration (self, priv->pipeline_mp3, FALSE);
+		return FALSE;
+
+	default:
+		break;
+	}
+
+	return TRUE;
+}
+
+static gboolean
+brasero_metadata_create_mp3_pipeline (BraseroMetadata *self)
+{
+	BraseroMetadataPrivate *priv;
+	GstElement *source;
+	GstElement *parse;
+	GstElement *sink;
+	GstBus *bus;
+
+	priv = BRASERO_METADATA_PRIVATE (self);
+
+	priv->pipeline_mp3 = gst_pipeline_new (NULL);
+
+	source = gst_element_make_from_uri (GST_URI_SRC,
+					    priv->info->uri,
+					    NULL);
+	if (!source) {
+		priv->error = g_error_new (BRASERO_UTILS_ERROR,
+					   BRASERO_UTILS_ERROR_GENERAL,
+					   _("%s element could not be created"),
+					   "\"Source\"");
+
+		g_object_unref (priv->pipeline_mp3);
+		priv->pipeline_mp3 = NULL;
+		return FALSE;
+	}
+	gst_bin_add (GST_BIN (priv->pipeline_mp3), source);
+
+	parse = gst_element_factory_make ("mp3parse", NULL);
+	if (!parse) {
+		priv->error = g_error_new (BRASERO_UTILS_ERROR,
+					   BRASERO_UTILS_ERROR_GENERAL,
+					   _("%s element could not be created"),
+					   "\"mp3parse\"");
+
+		g_object_unref (priv->pipeline_mp3);
+		priv->pipeline_mp3 = NULL;
+		return FALSE;
+	}
+	gst_bin_add (GST_BIN (priv->pipeline_mp3), parse);
+
+	sink = gst_element_factory_make ("fakesink", NULL);
+	if (!sink) {
+		priv->error = g_error_new (BRASERO_UTILS_ERROR,
+					   BRASERO_UTILS_ERROR_GENERAL,
+					   _("%s element could not be created"),
+					   "\"Fakesink\"");
+
+		g_object_unref (priv->pipeline_mp3);
+		priv->pipeline_mp3 = NULL;
+		return FALSE;
+	}
+	gst_bin_add (GST_BIN (priv->pipeline_mp3), sink);
+
+	/* Link */
+	if (!gst_element_link_many (source, parse, sink, NULL)) {
+		g_object_unref (priv->pipeline_mp3);
+		priv->pipeline_mp3 = NULL;
+		return FALSE;
+	}
+
+	/* Bus */
+	bus = gst_pipeline_get_bus (GST_PIPELINE (priv->pipeline_mp3));
+	priv->watch_mp3 = gst_bus_add_watch (bus,
+					     (GstBusFunc) brasero_metadata_mp3_bus_messages,
+					     self);
+	gst_object_unref (bus);
+
+	gst_element_set_state (priv->pipeline_mp3, GST_STATE_PLAYING);
+	return TRUE;
+}
+
+static gboolean
+brasero_metadata_success_main (BraseroMetadata *self)
+{
+	BraseroMetadataPrivate *priv;
+
+	priv = BRASERO_METADATA_PRIVATE (self);
+
+	BRASERO_UTILS_LOG ("Metadata retrieval successfully completed for %s", priv->info->uri);
+
+	/* find the type of the file */
+	brasero_metadata_get_mime_type (self);
+
+	/* empty the bus of any pending message */
+	brasero_metadata_process_pending_messages (self);
+
+	/* get the size */
+	if (brasero_metadata_is_mp3 (self)) {
+		if (!brasero_metadata_create_mp3_pipeline (self)) {
+			BRASERO_UTILS_LOG ("Impossible to run mp3 specific pipeline");
+			return brasero_metadata_completed (self);
+		}
+
+		/* Return FALSE here not because we failed but to stop the Bus callback */
+		return FALSE;
+	}
+
+	return brasero_metadata_get_duration (self, priv->pipeline, TRUE);
+}
+
 static gboolean
 brasero_metadata_bus_messages (GstBus *bus,
 			       GstMessage *msg,
@@ -1202,93 +1302,17 @@ brasero_metadata_bus_messages (GstBus *bus,
 
 	switch (GST_MESSAGE_TYPE (msg)) {
 	case GST_MESSAGE_ELEMENT:
-		/* This is for snapshot function */
-		if (!strcmp (gst_structure_get_name (msg->structure), "preroll-pixbuf")
-		||  !strcmp (gst_structure_get_name (msg->structure), "pixbuf")) {
-			const GValue *value;
-
-			value = gst_structure_get_value (msg->structure, "pixbuf");
-			priv->info->snapshot = g_value_get_object (value);
-			g_object_ref (priv->info->snapshot);
-
-			BRASERO_UTILS_LOG ("Received pixbuf snapshot sink (%p) for %s", priv->info->snapshot, priv->info->uri);
-
-			/* Now we can stop */
-			return brasero_metadata_completed (self);
-		}
-
-		/* here we just want to check if that's a missing codec */
-		if ((priv->flags & BRASERO_METADATA_FLAG_MISSING)
-		&&   gst_is_missing_plugin_message (msg))
-			priv->missing_plugins = g_slist_prepend (priv->missing_plugins, gst_message_ref (msg));
-		else if (!strcmp (gst_structure_get_name (msg->structure), "level")
-		&&   gst_structure_has_field (msg->structure, "peak")) {
-			const GValue *value;
-			const GValue *list;
-			gdouble peak;
-
-			list = gst_structure_get_value (msg->structure, "peak");
-			value = gst_value_list_get_value (list, 0);
-			peak = g_value_get_double (value);
-
-			/* detection of silence */
-			if (peak < -50.0) {
-				gint64 pos = -1;
-				GstFormat format = GST_FORMAT_TIME;
-	
-				/* was there a silence last time we check ?
-				 * NOTE: if that's the first signal we receive
-				 * then consider that silence started from 0 */
-				gst_element_query_position (priv->pipeline, &format, &pos);
-				if (pos == -1) {
-					BRASERO_UTILS_LOG ("impossible to retrieve position");
-					return TRUE;
-				}
-
-				if (!priv->silence) {
-					priv->silence = g_new0 (BraseroMetadataSilence, 1);
-					if (priv->prev_level_mes) {
-						priv->silence->start = pos;
-						priv->silence->end = pos;
-					}
-					else {
-						priv->silence->start = 0;
-						priv->silence->end = pos;
-					}
-				}				
-				else
-					priv->silence->end = pos;
-
-				BRASERO_UTILS_LOG ("silence detected at %lli", pos);
-			}
-			else if (priv->silence) {
-				BRASERO_UTILS_LOG ("silence finished");
-
-				priv->info->silences = g_slist_append (priv->info->silences,
-								       priv->silence);
-				priv->silence = NULL;
-			}
-			priv->prev_level_mes = 1;
-		}
-		break;
+		return brasero_metadata_process_element_messages (self, msg);
 
 	case GST_MESSAGE_ERROR:
 		/* save the error message */
 		gst_message_parse_error (msg, &error, &debug_string);
-		BRASERO_UTILS_LOG ("Gstreamer error (%s)", debug_string);
+		BRASERO_UTILS_LOG ("GStreamer error (%s)", debug_string);
 		g_free (debug_string);
 		if (!priv->error && error)
 			priv->error = error;
 
-		/* See if we have missing plugins */
-		if (priv->missing_plugins) {
-			if (!brasero_metadata_install_missing_plugins (self))
-				return brasero_metadata_completed (self);
-		}
-		else
-			return brasero_metadata_completed (self);
-
-		break;
+		return brasero_metadata_completed (self);
 
 	case GST_MESSAGE_EOS:
 		BRASERO_UTILS_LOG ("End of stream reached for %s", priv->info->uri);
@@ -1643,6 +1667,7 @@ brasero_metadata_new_decoded_pad_cb (GstElement *decode,
 		return;
 
 	name = gst_structure_get_name (structure);
+
 	has_audio = (g_strrstr (name, "audio") != NULL);
 	has_video = (g_strrstr (name, "video") != NULL);
 	priv->info->has_audio |= has_audio;
