@@ -438,24 +438,19 @@ brasero_readom_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static BraseroBurnResult
-brasero_readom_export_caps (BraseroPlugin *plugin, gchar **error)
+static void
+brasero_readom_export_caps (BraseroPlugin *plugin)
 {
-	BraseroBurnResult result;
 	GSList *output;
 	GSList *input;
 
 	brasero_plugin_define (plugin,
 			       "readom",
-			       _("Use readom to create disc images"),
+	                       NULL,
+			       _("Copies any disc to a disc image"),
 			       "Philippe Rouquier",
 			       1);
 
-	/* First see if this plugin can be used */
-	result = brasero_process_check_path ("readom", error);
-	if (result != BRASERO_BURN_OK)
-		return result;
-
 	/* that's for clone mode only The only one to copy audio */
 	output = brasero_caps_image_new (BRASERO_PLUGIN_IO_ACCEPT_FILE,
 					 BRASERO_IMAGE_FORMAT_CLONE);
@@ -497,6 +492,15 @@ brasero_readom_export_caps (BraseroPlugin *plugin, gchar **error)
 	g_slist_free (input);
 
 	brasero_plugin_register_group (plugin, _(CDRKIT_DESCRIPTION));
+}
 
-	return BRASERO_BURN_OK;
+G_MODULE_EXPORT void
+brasero_plugin_check_config (BraseroPlugin *plugin)
+{
+	gint version [3] = { 1, 1, 0};
+	brasero_plugin_test_app (plugin,
+	                         "readom",
+	                         "--version",
+	                         "readcd %*s is not what you see here. This line is only a fake for too clever\nGUIs and other frontend applications. In fact, this program is:\nreadom %d.%d.%d",
+	                         version);
 }
