@@ -932,8 +932,8 @@ brasero_libburn_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static BraseroBurnResult
-brasero_libburn_export_caps (BraseroPlugin *plugin, gchar **error)
+static void
+brasero_libburn_export_caps (BraseroPlugin *plugin)
 {
 	const BraseroMedia media_cd = BRASERO_MEDIUM_CD|
 				      BRASERO_MEDIUM_REWRITABLE|
@@ -970,7 +970,8 @@ brasero_libburn_export_caps (BraseroPlugin *plugin, gchar **error)
 
 	brasero_plugin_define (plugin,
 			       "libburn",
-			       _("libburn burns CD(RW), DVD+/-(RW)"),
+	                       NULL,
+			       _("Burns, blanks and formats CDs, DVDs and BDs"),
 			       "Philippe Rouquier",
 			       15);
 
@@ -995,7 +996,7 @@ brasero_libburn_export_caps (BraseroPlugin *plugin, gchar **error)
 	/* audio support for CDs only */
 	input = brasero_caps_audio_new (BRASERO_PLUGIN_IO_ACCEPT_PIPE|
 					BRASERO_PLUGIN_IO_ACCEPT_FILE,
-					BRASERO_AUDIO_FORMAT_RAW);
+					BRASERO_AUDIO_FORMAT_RAW_LITTLE_ENDIAN);
 	
 	output = brasero_caps_disc_new (media_cd);
 	brasero_plugin_link_caps (plugin, output, input);
@@ -1086,5 +1087,4 @@ brasero_libburn_export_caps (BraseroPlugin *plugin, gchar **error)
 					BRASERO_BURN_FLAG_NONE);
 
 	brasero_plugin_register_group (plugin, _(LIBBURNIA_DESCRIPTION));
-	return BRASERO_BURN_OK;
 }
